<?php

/**
  * Copyright (c) 2006-2018 Massachusetts General Hospital
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the i2b2 Software License v2.1
  * which accompanies this distribution.
  *
  * i2b2.php
  * servcie call wrappers for the PM Cell
  * @category ACT
  * @package  Registration 
  * @author   Nich and Jay
  * @version  May 2018
  */



function i2b2_header_xml($uri, $domain, $username, $password, $project="SHRINE_ACT"){
$xml = <<<XML
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<i2b2:request xmlns:i2b2="http://www.i2b2.org/xsd/hive/msg/1.1/" xmlns:pm="http://www.i2b2.org/xsd/cell/pm/1.1/">
    <message_header>
        <proxy>
            <redirect_url>I2B2_PM_URI</redirect_url>
        </proxy>

        <i2b2_version_compatible>1.1</i2b2_version_compatible>
        <hl7_version_compatible>2.4</hl7_version_compatible>
        <sending_application>
            <application_name>i2b2 Project Management</application_name>
            <application_version>1.1</application_version>
        </sending_application>
        <sending_facility>
            <facility_name>i2b2 Hive</facility_name>
        </sending_facility>
        <receiving_application>
            <application_name>Project Management Cell</application_name>
            <application_version>1.1</application_version>
        </receiving_application>
        <receiving_facility>
            <facility_name>i2b2 Hive</facility_name>
        </receiving_facility>
        <datetime_of_message>2007-04-09T15:19:18.906-04:00</datetime_of_message>
		<security>
			<domain>I2B2_DOMAIN</domain>
			<username>I2B2_SERVICE_ACCOUNT_ID</username>
			<password>I2B2_SERVICE_ACCOUNT_PW</password>
		</security>
        <message_control_id>
            <message_num>2pNloq58C28eP511z8nkB</message_num>
            <instance_num>0</instance_num>
        </message_control_id>
        <processing_id>
            <processing_id>P</processing_id>
            <processing_mode>I</processing_mode>
        </processing_id>
        <accept_acknowledgement_type>AL</accept_acknowledgement_type>
        <application_acknowledgement_type>AL</application_acknowledgement_type>
        <country_code>US</country_code>
        <project_id>I2B2_PROJECT</project_id>
    </message_header>
    <request_header>
        <result_waittime_ms>180000</result_waittime_ms>
    </request_header>
    <message_body>
XML;

	$xml = str_replace("I2B2_PM_URI", $uri, $xml);
	$xml = str_replace("I2B2_DOMAIN", $domain, $xml);
	$xml = str_replace("I2B2_SERVICE_ACCOUNT_ID", $username, $xml);
	$xml = str_replace("I2B2_SERVICE_ACCOUNT_PW", $password, $xml);
	$xml = str_replace("I2B2_PROJECT", $project, $xml);
	
	return $xml;
}

function i2b2_footer_xml(){

$xml = <<<XML2
</message_body>
</i2b2:request>
XML2;
	
return $xml;

}

function i2b2_get_user($user_name){
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:get_user>$user_name</pm:get_user>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
	return $data;
}

function i2b2_get_all_user_param($user_name){
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:get_all_user_param>$user_name</pm:get_all_user_param>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
	return $data;
}

function i2b2_get_role($user_name, $project_id){
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:get_role>";
	$request_xml .= "	<project_id>$project_id</project_id>";
	$request_xml .= "	<user_name>$user_name</user_name>";
	$request_xml .= "</pm:get_role>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);		
	return $data;
}

function i2b2_set_user($user_name, $full_name, $email, $status, $password){
	/*
	 * Construct set_user XML call and attach to message_body parent
	 * <message_body>
	 *  <pm:set_user>
	 *      <user_name>mikem</user_name>
	 *      <full_name>Mike M</full_name>
	 *      <email>a@b.com</email>
	 *      <status_cd>A</status_cd>
	 *      <password>tiger</password>
	 *  </pm:set_user>
	 * </message_body>
	 * 
	 * Returns: Boolean true when user is set ("SUCCESS")
	 */

	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:set_user>";
	$request_xml .= "	<user_name>$user_name</user_name>";
	$request_xml .= "	<full_name>$full_name</full_name>";
	$request_xml .= "	<email>$email</email>";
	$request_xml .= "	<status_cd>A</status_cd>";
	$request_xml .= "	<password>tiger</password>";
	$request_xml .= "</pm:set_user>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
	
	
	return $data;
		 
}



function i2b2_delete_user($user_name){
	/*
	 * Construct delete_user XML call and attach to message_body parent
	 * <message_body>
	 *  <pm:delete_user>mikem</delete_user>
	 * </message_body>
	 * 
	 * Returns: Boolean true when user is deleted
	 */
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:delete_user>$user_name</pm:delete_user>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
		
	return $data;
}


function i2b2_set_role($user_name, $role, $project_id){
	/*
	 * Construct set_role XML call and attach to message_body parent
	 * <pm:set_role>
	 *  <user_name>mem61</user_name>
	 *  <role>DATA_AGG</role>
	 *  <project_id>ra_mart_test2</project_id>
	 * </pm:set_role>
	 * 
	 * Returns: Boolean true when user role is set
	 */
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:set_role>";
	$request_xml .= "	<user_name>$user_name</user_name>";
	$request_xml .= "	<role>$role</role>";
	$request_xml .= "	<project_id>$project_id</project_id>";
	$request_xml .= "</pm:set_role>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
		
	return $data;
		 
}



function i2b2_delete_role($user_name, $role, $project_id){
	/*
	 * Construct set_role XML call and attach to message_body parent
	 * <pm:delete_role>
	 *  <user_name>mem61</user_name>
	 *  <role>DATA_AGG</role>
	 *  <project_id>ra_mart_test2</project_id>
	 * </pm:delete_role>
	 * 
	 * Returns: Boolean true when user role is set
	 */
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:delete_role>";
	$request_xml .= "	<user_name>$user_name</user_name>";
	$request_xml .= "	<role>$role</role>";
	$request_xml .= "	<project_id>$project_id</project_id>";
	$request_xml .= "</pm:delete_role>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
		
	return $data;
		 
}

function i2b2_set_user_param($user_name, $param_name, $param_type, $param_value){
	/*
	 * Construct set_role XML call and attach to message_body parent
	 * <pm:set_user_param>
	 *  <user_name>snm0</user_name>
	 *  <param name="tests" datatype="T" id="123">myvalue2</param>
	 * </pm:set_user_param>
	 * 
	 * Returns: Boolean true when user role is set
	 */
	global $config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw;
	
	$request_xml = i2b2_header_xml($config_pm_uri, $config_domain, $config_service_account_id, $config_service_account_pw);
	$request_xml .= "<pm:set_user_param>";
	$request_xml .= "	<user_name>$user_name</user_name>";
	$request_xml .= "	<param name=\"$param_name\" datatype=\"$param_type\">$param_value</param>";
	$request_xml .= "</pm:set_user_param>";
	$request_xml .= i2b2_footer_xml();
	
	$ch = curl_init($config_pm_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: text/xml'));
	curl_setopt($ch, CURLOPT_POSTFIELDS, "$request_xml");
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$data = curl_exec($ch);
	
	return $data;
		 
}


function result_status_error($xml_resp){
	if(preg_match("/<status type=\"ERROR\">/i", $xml_resp, $match)){
		return true;
	}
	return false;

}

function user_has_role($role, $xml_resp){
	
	if(preg_match("/<role>$role<\/role>/", $xml_resp, $match)){
		return true;
	}
	return false;

}

function print_roles($xml_resp){
	
	$roles = '';
	preg_match_all("/<role>(.*)<\/role>/i", $xml_resp, $match);
	for($i=0; $i<count($match[1]); $i++){
		$roles .= $match[1][$i];
		if($i !== count($match[1])-1){
			$roles .= ', ';
		}
	}
	if($roles == '') return 'None';
	return $roles;	

}


?>
